@@ -1,11 +1,11 @@
-.fnn_ts_method = \(data, target, lib = NULL, pred = NULL, E = 2:10,
-                   tau = 0, rt = 10, eps = 2, threads = length(E)){
+.fnn_ts_method = \(data, target, lib = NULL, pred = NULL, E = 2:10, tau = 1, 
+                   dist.metric = "L1", rt = 10, eps = 2, threads = length(E)){
   vec = .uni_ts(data,target)
   rt = .check_inputelementnum(rt,max(E))
   eps = .check_inputelementnum(eps,max(E))
   if (is.null(lib)) lib = which(!is.na(vec))
   if (is.null(pred)) pred = lib
-  return(RcppFNN4TS(vec,rt,eps,lib,pred,E,tau,threads))
+  return(RcppFNN4TS(vec,rt,eps,lib,pred,E,tau,.check_distmetric(dist.metric),threads))
 }
 
 #' false nearest neighbours
@@ -13,6 +13,7 @@
 #' @inheritParams embedded
 #' @param lib (optional) libraries indices.
 #' @param pred (optional) predictions indices.
+#' @param dist.metric (optional) distance metric (`L1`: Manhattan, `L2`: Euclidean).
 #' @param rt (optional) escape factor.
 #' @param eps (optional) neighborhood diameter.
 #' @param threads (optional) number of threads to use.
