@@ -71,13 +71,14 @@ Rcpp::List RcppLogisticMap(
 // Wrapper function to generate time-delay embeddings for a univariate time series
 // [[Rcpp::export(rng = false)]]
 Rcpp::NumericMatrix RcppEmbed(const Rcpp::NumericVector& vec,
-                              int E,
-                              int tau = 0) {
+                              int E = 3,
+                              int tau = 1,
+                              int style = 0) {
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> vec_std = Rcpp::as<std::vector<double>>(vec);
 
   // Generate embeddings
-  std::vector<std::vector<double>> embeddings = Embed(vec_std, E, tau);
+  std::vector<std::vector<double>> embeddings = Embed(vec_std, E, tau, style);
 
   // Convert std::vector<std::vector<double>> to Rcpp::NumericMatrix
   int rows = embeddings.size();
@@ -101,7 +102,9 @@ Rcpp::NumericVector RcppSimplexForecast(
     int tau,
     const Rcpp::IntegerVector& lib,
     const Rcpp::IntegerVector& pred,
-    const int& num_neighbors){
+    const int& num_neighbors,
+    const int& dist_metric,
+    const bool& dist_average){
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> source_std = Rcpp::as<std::vector<double>>(source);
   std::vector<double> target_std = Rcpp::as<std::vector<double>>(target);
@@ -114,7 +117,7 @@ Rcpp::NumericVector RcppSimplexForecast(
   std::vector<int> pred_indices;
 
   int target_len = target_std.size();
-  int max_lag = (tau == 0) ? (E - 1) : (E * tau);
+  int max_lag = (tau == 0) ? (E - 1) : ((E - 1) * tau);
   // Convert lib and pred (1-based in R) to 0-based indices and check validity
   for (int i = 0; i < lib.size(); ++i) {
     if (lib[i] < 0 || lib[i] > target_len) {
@@ -122,7 +125,7 @@ Rcpp::NumericVector RcppSimplexForecast(
     }
     if (!std::isnan(source_std[lib[i] - 1]) &&
         !std::isnan(target_std[lib[i] - 1]) &&
-        (lib[i] > max_lag + 1)) {
+        (lib[i] > max_lag)) {
       lib_indices.push_back(lib[i] - 1); // Convert to 0-based index
     }
   }
@@ -131,8 +134,7 @@ Rcpp::NumericVector RcppSimplexForecast(
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
     if (!std::isnan(source_std[pred[i] - 1]) &&
-        !std::isnan(target_std[pred[i] - 1]) &&
-        (pred[i] > max_lag + 1)) {
+        !std::isnan(target_std[pred[i] - 1])) {
       pred_indices.push_back(pred[i] - 1); // Convert to 0-based index
     }
   }
@@ -143,7 +145,9 @@ Rcpp::NumericVector RcppSimplexForecast(
     target_std,
     lib_indices,
     pred_indices,
-    num_neighbors
+    num_neighbors,
+    dist_metric,
+    dist_average
   );
 
   // Convert the result back to Rcpp::NumericVector
@@ -160,7 +164,9 @@ Rcpp::NumericVector RcppSMapForecast(
     const Rcpp::IntegerVector& lib,
     const Rcpp::IntegerVector& pred,
     const int& num_neighbors,
-    const double& theta){
+    const double& theta,
+    const int& dist_metric,
+    const bool& dist_average){
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> source_std = Rcpp::as<std::vector<double>>(source);
   std::vector<double> target_std = Rcpp::as<std::vector<double>>(target);
@@ -173,7 +179,7 @@ Rcpp::NumericVector RcppSMapForecast(
   std::vector<int> pred_indices;
 
   int target_len = target_std.size();
-  int max_lag = (tau == 0) ? (E - 1) : (E * tau);
+  int max_lag = (tau == 0) ? (E - 1) : ((E - 1) * tau);
   // Convert lib and pred (1-based in R) to 0-based indices and check validity
   for (int i = 0; i < lib.size(); ++i) {
     if (lib[i] < 0 || lib[i] > target_len) {
@@ -181,7 +187,7 @@ Rcpp::NumericVector RcppSMapForecast(
     }
     if (!std::isnan(source_std[lib[i] - 1]) &&
         !std::isnan(target_std[lib[i] - 1]) &&
-        (lib[i] > max_lag + 1)) {
+        (lib[i] > max_lag)) {
       lib_indices.push_back(lib[i] - 1); // Convert to 0-based index
     }
   }
@@ -190,8 +196,7 @@ Rcpp::NumericVector RcppSMapForecast(
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
     if (!std::isnan(source_std[pred[i] - 1]) &&
-        !std::isnan(target_std[pred[i] - 1]) &&
-        (pred[i] > max_lag + 1)) {
+        !std::isnan(target_std[pred[i] - 1])) {
       pred_indices.push_back(pred[i] - 1); // Convert to 0-based index
     }
   }
@@ -203,7 +208,9 @@ Rcpp::NumericVector RcppSMapForecast(
     lib_indices,
     pred_indices,
     num_neighbors,
-    theta
+    theta,
+    dist_metric,
+    dist_average
   );
 
   // Convert the result back to Rcpp::NumericVector
@@ -221,6 +228,7 @@ Rcpp::NumericVector RcppIntersectionCardinality(
     const Rcpp::IntegerVector& pred,
     const int& num_neighbors = 4,
     const int& n_excluded = 0,
+    const int& dist_metric = 2,
     const int& threads = 8,
     const int& parallel_level = 0){
   // Convert Rcpp::NumericVector to std::vector<double>
@@ -236,7 +244,7 @@ Rcpp::NumericVector RcppIntersectionCardinality(
   std::vector<size_t> pred_indices;
 
   int target_len = target_std.size();
-  int max_lag = (tau == 0) ? (E - 1) : (E * tau);
+  int max_lag = (tau == 0) ? (E - 1) : ((E - 1) * tau);
   // Convert lib and pred (1-based in R) to 0-based indices and check validity
   for (int i = 0; i < lib.size(); ++i) {
     if (lib[i] < 0 || lib[i] > target_len) {
@@ -244,7 +252,7 @@ Rcpp::NumericVector RcppIntersectionCardinality(
     }
     if (!std::isnan(source_std[lib[i] - 1]) &&
         !std::isnan(target_std[lib[i] - 1]) &&
-        (lib[i] > max_lag + 1)){
+        (lib[i] > max_lag)){
         lib_indices.push_back(static_cast<size_t>(lib[i] - 1)); // Convert to 0-based index
     }
   }
@@ -253,8 +261,7 @@ Rcpp::NumericVector RcppIntersectionCardinality(
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
     if (!std::isnan(source_std[pred[i] - 1]) &&
-        !std::isnan(target_std[pred[i] - 1]) &&
-        (pred[i] > max_lag + 1)){
+        !std::isnan(target_std[pred[i] - 1])){
         pred_indices.push_back(static_cast<size_t>(pred[i] - 1)); // Convert to 0-based index
     }
   }
@@ -271,6 +278,7 @@ Rcpp::NumericVector RcppIntersectionCardinality(
     pred_indices,
     static_cast<size_t>(num_neighbors),
     static_cast<size_t>(n_excluded),
+    dist_metric,
     threads,
     parallel_level
   );
@@ -285,12 +293,14 @@ Rcpp::NumericVector RcppMVE4TS(const Rcpp::NumericMatrix& x,
                                const Rcpp::NumericVector& y,
                                const Rcpp::IntegerVector& lib,
                                const Rcpp::IntegerVector& pred,
-                               int E,
-                               int tau,
-                               int b,
-                               int top,
-                               int nvar,
-                               int threads){
+                               int E = 3,
+                               int tau = 1,
+                               int b = 4,
+                               int top = 3,
+                               int nvar = 3,
+                               int dist_metric = 2,
+                               int dist_average = true,
+                               int threads = 8){
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> target = Rcpp::as<std::vector<double>>(y);
 
@@ -306,7 +316,7 @@ Rcpp::NumericVector RcppMVE4TS(const Rcpp::NumericMatrix& x,
     if (lib[i] < 1 || lib[i] > target_len) {
       Rcpp::stop("lib contains out-of-bounds index at position %d (value: %d)", i + 1, lib[i]);
     }
-    if (!std::isnan(target[lib[i] - 1]) && (lib[i] > max_lag + 1)) {
+    if (!std::isnan(target[lib[i] - 1]) && (lib[i] > max_lag)) {
       lib_indices.push_back(lib[i] - 1); // Convert to 0-based index
     }
   }
@@ -315,7 +325,7 @@ Rcpp::NumericVector RcppMVE4TS(const Rcpp::NumericMatrix& x,
     if (pred[i] < 1 || pred[i] > target_len) {
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
-    if (!std::isnan(target[pred[i] - 1]) && (pred[i] > max_lag + 1)) {
+    if (!std::isnan(target[pred[i] - 1])) {
       pred_indices.push_back(pred[i] - 1); // Convert to 0-based index
     }
   }
@@ -389,6 +399,8 @@ Rcpp::NumericVector RcppMVE4TS(const Rcpp::NumericMatrix& x,
     pred_indices,
     b,
     k,
+    dist_metric,
+    dist_average,
     threads);
 
   // Convert the result back to Rcpp::NumericVector
@@ -404,7 +416,8 @@ Rcpp::NumericVector RcppFNN4TS(
     const Rcpp::IntegerVector& lib,
     const Rcpp::IntegerVector& pred,
     const Rcpp::IntegerVector& E,
-    int tau = 0,
+    int tau = 1,
+    int dist_metric = 2,
     int threads = 8,
     int parallel_level = 0){
   // Convert Rcpp::NumericVector to std::vector<double>
@@ -417,18 +430,18 @@ Rcpp::NumericVector RcppFNN4TS(
   std::vector<int> pred_std;
 
   // Generate embeddings
-  std::vector<double> E_std = Rcpp::as<std::vector<double>>(E);
-  int max_E = CppMax(E_std, true);
+  std::vector<int> E_std = Rcpp::as<std::vector<int>>(E);
+  int max_E = *std::max_element(E_std.begin(), E_std.end());
+  int max_lag = (tau == 0) ? (max_E - 1) : ((max_E - 1) * tau);
   std::vector<std::vector<double>> embeddings = Embed(vec_std, max_E, tau);
 
   int validSampleNum = vec_std.size();
-  int max_lag = (tau == 0) ? (max_E - 1) : (max_E * tau);
   // Check that lib and pred indices are within bounds & convert R based 1 index to C++ based 0 index
   for (int i = 0; i < lib.size(); ++i) {
     if (lib[i] < 1 || lib[i] > validSampleNum) {
       Rcpp::stop("lib contains out-of-bounds index at position %d (value: %d)", i + 1, lib[i]);
     }
-    if (!std::isnan(vec_std[lib[i] - 1]) && (lib[i] > max_lag + 1)) {
+    if (!std::isnan(vec_std[lib[i] - 1]) && (lib[i] > max_lag)) {
       lib_std.push_back(lib[i] - 1);
     }
   }
@@ -436,13 +449,16 @@ Rcpp::NumericVector RcppFNN4TS(
     if (pred[i] < 1 || pred[i] > validSampleNum) {
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
-    if (!std::isnan(vec_std[pred[i] - 1]) && (pred[i] > max_lag + 1)) {
+    if (!std::isnan(vec_std[pred[i] - 1])) {
       pred_std.push_back(pred[i] - 1);
     }
   }
+  
+  // Use L1 norm (Manhattan distance) if dist_metric == 1, else use L2 norm
+  bool L1norm = (dist_metric == 1);
 
   // Perform FNN for time series data
-  std::vector<double> fnn = CppFNN(embeddings,lib_std,pred_std,rt_std,eps_std,true,threads,parallel_level);
+  std::vector<double> fnn = CppFNN(embeddings,lib_std,pred_std,rt_std,eps_std,L1norm,threads,parallel_level);
 
   // Convert the result back to Rcpp::NumericVector and set names as "E:1", "E:2", ..., "E:n"
   Rcpp::NumericVector result = Rcpp::wrap(fnn);
@@ -463,7 +479,9 @@ Rcpp::NumericMatrix RcppSimplex4TS(const Rcpp::NumericVector& source,
                                    const Rcpp::IntegerVector& pred,
                                    const Rcpp::IntegerVector& E,
                                    const Rcpp::IntegerVector& b,
-                                   int tau = 0,
+                                   int tau = 1,
+                                   int dist_metric = 2,
+                                   bool dist_average = true,
                                    int threads = 8) {
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> source_std = Rcpp::as<std::vector<double>>(source);
@@ -476,6 +494,8 @@ Rcpp::NumericMatrix RcppSimplex4TS(const Rcpp::NumericVector& source,
   // Initialize lib_indices and pred_indices
   std::vector<int> lib_indices;
   std::vector<int> pred_indices;
+  int max_E = *std::max_element(E_std.begin(), E_std.end());
+  int max_lag = (tau == 0) ? (max_E - 1) : ((max_E - 1) * tau);
 
   int target_len = target_std.size();
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
@@ -485,7 +505,8 @@ Rcpp::NumericMatrix RcppSimplex4TS(const Rcpp::NumericVector& source,
       Rcpp::stop("lib contains out-of-bounds index at position %d (value: %d)", i + 1, lib[i]);
     }
     if (!std::isnan(source_std[lib[i] - 1]) &&
-        !std::isnan(target_std[lib[i] - 1])) {
+        !std::isnan(target_std[lib[i] - 1]) &&
+        (lib[i] > max_lag)) {
       lib_indices.push_back(lib[i] - 1); // Convert to 0-based index
     }
   }
@@ -508,6 +529,8 @@ Rcpp::NumericMatrix RcppSimplex4TS(const Rcpp::NumericVector& source,
     E_std,
     b_std,
     tau,
+    dist_metric,
+    dist_average,
     threads);
 
   size_t n_rows = res_std.size();
@@ -536,8 +559,10 @@ Rcpp::NumericMatrix RcppSMap4TS(const Rcpp::NumericVector& source,
                                 const Rcpp::IntegerVector& pred,
                                 const Rcpp::NumericVector& theta,
                                 int E = 3,
-                                int tau = 0,
+                                int tau = 1,
                                 int b = 4,
+                                int dist_metric = 2,
+                                bool dist_average = true,
                                 int threads = 8) {
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> source_std = Rcpp::as<std::vector<double>>(source);
@@ -549,7 +574,7 @@ Rcpp::NumericMatrix RcppSMap4TS(const Rcpp::NumericVector& source,
   std::vector<int> pred_indices;
 
   int target_len = target_std.size();
-  int max_lag = (tau == 0) ? (E - 1) : (E * tau);
+  int max_lag = (tau == 0) ? (E - 1) : ((E - 1) * tau);
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
   size_t n_libsize = lib.size();   // convert R R_xlen_t to C++ size_t
   for (size_t i = 0; i < n_libsize; ++i) {
@@ -558,7 +583,7 @@ Rcpp::NumericMatrix RcppSMap4TS(const Rcpp::NumericVector& source,
     }
     if (!std::isnan(source_std[lib[i] - 1]) &&
         !std::isnan(target_std[lib[i] - 1]) &&
-        (lib[i] > max_lag + 1)) {
+        (lib[i] > max_lag)) {
       lib_indices.push_back(lib[i] - 1); // Convert to 0-based index
     }
   }
@@ -568,8 +593,7 @@ Rcpp::NumericMatrix RcppSMap4TS(const Rcpp::NumericVector& source,
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
     if (!std::isnan(source_std[pred[i] - 1]) &&
-        !std::isnan(target_std[pred[i] - 1]) &&
-        (pred[i] > max_lag + 1)) {
+        !std::isnan(target_std[pred[i] - 1])) {
       pred_indices.push_back(pred[i] - 1); // Convert to 0-based index
     }
   }
@@ -583,6 +607,8 @@ Rcpp::NumericMatrix RcppSMap4TS(const Rcpp::NumericVector& source,
     E,
     tau,
     b,
+    dist_metric,
+    dist_average,
     threads);
 
   size_t n_rows = res_std.size();
@@ -611,7 +637,9 @@ Rcpp::NumericMatrix RcppMultiSimplex4TS(const Rcpp::NumericMatrix& source,
                                         const Rcpp::IntegerVector& pred,
                                         const Rcpp::IntegerVector& E,
                                         const Rcpp::IntegerVector& b,
-                                        int tau = 0,
+                                        int tau = 1,
+                                        int dist_metric = 2,
+                                        bool dist_average = true,
                                         int threads = 8) {
   // Convert Rcpp NumericMatrix to std::vector of std::vectors
   std::vector<std::vector<double>> source_std(source.ncol());
@@ -681,6 +709,8 @@ Rcpp::NumericMatrix RcppMultiSimplex4TS(const Rcpp::NumericMatrix& source,
     E_std,
     b_std,
     tau,
+    dist_metric,
+    dist_average,
     threads);
 
   size_t n_rows = res_std.size();
@@ -709,8 +739,9 @@ Rcpp::NumericMatrix RcppIC4TS(const Rcpp::NumericVector& source,
                               const Rcpp::IntegerVector& pred,
                               const Rcpp::IntegerVector& E,
                               const Rcpp::IntegerVector& b,
-                              int tau = 0,
+                              int tau = 1,
                               int exclude = 0,
+                              int dist_metric = 2,
                               int threads = 8,
                               int parallel_level = 0) {
   // Convert Rcpp::NumericVector to std::vector<double>
@@ -719,6 +750,8 @@ Rcpp::NumericMatrix RcppIC4TS(const Rcpp::NumericVector& source,
 
   // Convert Rcpp::IntegerVector to std::vector<int>
   std::vector<int> E_std = Rcpp::as<std::vector<int>>(E);
+  int max_E = *std::max_element(E_std.begin(), E_std.end());
+  int max_lag = (tau == 0) ? (max_E - 1) : ((max_E - 1) * tau);
 
   // Initialize lib_indices and pred_indices
   std::vector<size_t> lib_indices;
@@ -731,7 +764,9 @@ Rcpp::NumericMatrix RcppIC4TS(const Rcpp::NumericVector& source,
     if (lib[i] < 1 || lib[i] > target_len) {
       Rcpp::stop("lib contains out-of-bounds index at position %d (value: %d)", i + 1, lib[i]);
     }
-    if (!std::isnan(source_std[lib[i] - 1]) && !std::isnan(target_std[lib[i] - 1])) {
+    if (!std::isnan(source_std[lib[i] - 1]) &&
+        !std::isnan(target_std[lib[i] - 1]) &&
+        (lib[i] > max_lag)) {
       lib_indices.push_back(static_cast<size_t>(lib[i] - 1)); // Convert to 0-based index
     }
   }
@@ -740,7 +775,8 @@ Rcpp::NumericMatrix RcppIC4TS(const Rcpp::NumericVector& source,
     if (pred[i] < 1 || pred[i] > target_len) {
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
-    if (!std::isnan(source_std[pred[i] - 1]) && !std::isnan(target_std[pred[i] - 1])) {
+    if (!std::isnan(source_std[pred[i] - 1]) &&
+        !std::isnan(target_std[pred[i] - 1])) {
       pred_indices.push_back(static_cast<size_t>(pred[i] - 1)); // Convert to 0-based index
     }
   }
@@ -763,6 +799,7 @@ Rcpp::NumericMatrix RcppIC4TS(const Rcpp::NumericVector& source,
     b_std,
     tau,
     exclude,
+    dist_metric,
     threads,
     parallel_level);
 
@@ -799,6 +836,8 @@ Rcpp::NumericMatrix RcppCCM(const Rcpp::NumericVector& x,
                             double theta = 0,
                             int threads = 8,
                             int parallel_level = 0,
+                            int dist_metric = 2,
+                            bool dist_average = true,
                             bool progressbar = false) {
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> x_std = Rcpp::as<std::vector<double>>(x);
@@ -811,14 +850,14 @@ Rcpp::NumericMatrix RcppCCM(const Rcpp::NumericVector& x,
 
   // Check that lib and pred indices are within bounds & convert R based 1 index to C++ based 0 index
   int n = y_std.size();
-  int max_lag = (tau == 0) ? (E - 1) : (E * tau);
+  int max_lag = (tau == 0) ? (E - 1) : ((E - 1) * tau);
   for (int i = 0; i < lib.size(); ++i) {
     if (lib[i] < 1 || lib[i] > n) {
       Rcpp::stop("lib contains out-of-bounds index at position %d (value: %d)", i + 1, lib[i]);
     }
     if (!std::isnan(x_std[lib[i] - 1]) &&
         !std::isnan(y_std[lib[i] - 1]) &&
-        (lib[i] > max_lag + 1)) {
+        (lib[i] > max_lag)) {
       lib_std.push_back(lib[i] - 1);
     }
   }
@@ -827,8 +866,7 @@ Rcpp::NumericMatrix RcppCCM(const Rcpp::NumericVector& x,
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
     if (!std::isnan(x_std[pred[i] - 1]) &&
-        !std::isnan(y_std[pred[i] - 1]) &&
-        (pred[i] > max_lag + 1)) {
+        !std::isnan(y_std[pred[i] - 1])) {
       pred_std.push_back(pred[i] - 1);
     }
   }
@@ -847,6 +885,8 @@ Rcpp::NumericMatrix RcppCCM(const Rcpp::NumericVector& x,
     theta,
     threads,
     parallel_level,
+    dist_metric,
+    dist_average,
     progressbar);
 
   // Convert std::vector<std::vector<double>> to Rcpp::NumericMatrix
@@ -883,6 +923,8 @@ Rcpp::NumericMatrix RcppPCM(const Rcpp::NumericVector& x,
                             int threads = 8,
                             int parallel_level = 0,
                             bool cumulate = false,
+                            int dist_metric = 2,
+                            bool dist_average = true,
                             bool progressbar = false) {
   // Convert Rcpp::NumericVector to std::vector<double>
   std::vector<double> x_std = Rcpp::as<std::vector<double>>(x);
@@ -906,7 +948,15 @@ Rcpp::NumericMatrix RcppPCM(const Rcpp::NumericVector& x,
   std::vector<int> pred_std;
   int max_E = *std::max_element(E_std.begin(), E_std.end());
   int max_tau = *std::max_element(tau_std.begin(), tau_std.end());
-  int max_lag = (max_tau == 0) ? (max_E - 1) : (max_E * max_tau);
+  int max_lag = (max_tau == 0) ? (max_E - 1) : ((max_E - 1) * max_tau);
+
+  // Constrain the usable library scope to exclude untrusted predictions.
+  if (cumulate){
+    max_lag = max_lag * static_cast<int>(z.ncol());
+  } else {
+    max_lag = max_lag * 2;
+  }
+
   int n = y_std.size();
   for (int i = 0; i < lib.size(); ++i) {
     if (lib[i] < 1 || lib[i] > n) {
@@ -914,7 +964,7 @@ Rcpp::NumericMatrix RcppPCM(const Rcpp::NumericVector& x,
     }
     if (!std::isnan(x_std[lib[i] - 1]) &&
         !std::isnan(y_std[lib[i] - 1]) &&
-        (lib[i] > max_lag + 1)) {
+        (lib[i] > max_lag)) {
       lib_std.push_back(lib[i] - 1);
     }
   }
@@ -923,8 +973,7 @@ Rcpp::NumericMatrix RcppPCM(const Rcpp::NumericVector& x,
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
     if (!std::isnan(x_std[pred[i] - 1]) &&
-        !std::isnan(y_std[pred[i] - 1]) &&
-        (pred[i] > max_lag + 1)) {
+        !std::isnan(y_std[pred[i] - 1])) {
       pred_std.push_back(pred[i] - 1);
     }
   }
@@ -945,6 +994,8 @@ Rcpp::NumericMatrix RcppPCM(const Rcpp::NumericVector& x,
     threads,
     parallel_level,
     cumulate,
+    dist_metric,
+    dist_average,
     progressbar);
 
   // Convert std::vector<std::vector<double>> to Rcpp::NumericMatrix
@@ -979,8 +1030,9 @@ Rcpp::List RcppCMC(
     const Rcpp::IntegerVector& pred,
     const Rcpp::IntegerVector& E,
     const Rcpp::IntegerVector& tau,
-    int b,
+    int b = 4,
     int r = 0,
+    int dist_metric = 2,
     int threads = 8,
     int parallel_level = 0,
     bool progressbar = false){
@@ -993,6 +1045,10 @@ Rcpp::List RcppCMC(
   std::vector<int> E_std = Rcpp::as<std::vector<int>>(E);
   std::vector<int> tau_std = Rcpp::as<std::vector<int>>(tau);
 
+  int max_E = *std::max_element(E_std.begin(), E_std.end());
+  int max_tau = *std::max_element(tau_std.begin(), tau_std.end());
+  int max_lag = (max_tau == 0) ? (max_E - 1) : ((max_E - 1) * max_tau);
+
   int validSampleNum = x_std.size();
   // Convert and check that lib and pred indices are within bounds & convert R based 1 index to C++ based 0 index
   std::vector<size_t> lib_std;
@@ -1001,7 +1057,9 @@ Rcpp::List RcppCMC(
     if (lib[i] < 1 || lib[i] > validSampleNum) {
       Rcpp::stop("lib contains out-of-bounds index at position %d (value: %d)", i + 1, lib[i]);
     }
-    if (!std::isnan(x_std[lib[i] - 1]) && !std::isnan(y_std[lib[i] - 1])) {
+    if (!std::isnan(x_std[lib[i] - 1]) &&
+        !std::isnan(y_std[lib[i] - 1]) &&
+        (lib[i] > max_lag)) {
       lib_std.push_back(static_cast<size_t>(lib[i] - 1));
     }
   }
@@ -1009,7 +1067,8 @@ Rcpp::List RcppCMC(
     if (pred[i] < 1 || pred[i] > validSampleNum) {
       Rcpp::stop("pred contains out-of-bounds index at position %d (value: %d)", i + 1, pred[i]);
     }
-    if (!std::isnan(x_std[pred[i] - 1]) && !std::isnan(y_std[pred[i] - 1])) {
+    if (!std::isnan(x_std[pred[i] - 1]) &&
+        !std::isnan(y_std[pred[i] - 1])) {
       pred_std.push_back(static_cast<size_t>(pred[i] - 1));
     }
   }
@@ -1028,7 +1087,7 @@ Rcpp::List RcppCMC(
   // Perform CMC for time series data
   CMCRes res = CMC(e1,e2,libsizes_std,lib_std,pred_std,
                    static_cast<size_t>(b),static_cast<size_t>(r),
-                   threads,parallel_level,progressbar);
+                   dist_metric,threads,parallel_level,progressbar);
 
   // Convert mean_aucs to Rcpp::DataFrame
   std::vector<double> libs, aucs;
@@ -1070,6 +1129,8 @@ Rcpp::NumericMatrix RcppMultispatialCCM(const Rcpp::NumericMatrix& x,
                                         int seed = 42,
                                         int threads = 8,
                                         int parallel_level = 0,
+                                        int dist_metric = 2,
+                                        bool dist_average = true,
                                         bool progressbar = false) {
   // Convert Rcpp NumericMatrix to std::vector of std::vectors
   std::vector<std::vector<double>> x_std(x.ncol());
@@ -1098,6 +1159,8 @@ Rcpp::NumericMatrix RcppMultispatialCCM(const Rcpp::NumericMatrix& x,
     threads,
     std::abs(seed),
     parallel_level,
+    dist_metric,
+    dist_average,
     progressbar);
 
   // Convert std::vector<std::vector<double>> to Rcpp::NumericMatrix
