@@ -20,6 +20,8 @@
  *   - pred_indices: A vector of indices indicating the prediction set.
  *   - num_neighbors: Number of neighbors used for simplex projection.
  *   - top_num: Number of top-performing reconstructions to select.
+ *   - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean). 
+ *   - dist_average: Whether to average distance by the number of valid vector components.
  *   - threads: Number of threads used from the global pool.
  *
  * Returns:
@@ -30,9 +32,11 @@ std::vector<double> MVE(
     const std::vector<double>& target,
     const std::vector<int>& lib_indices,
     const std::vector<int>& pred_indices,
-    int num_neighbors,
-    int top_num,
-    int threads
+    int num_neighbors = 4,
+    int top_num = 3,
+    int dist_metric = 2,
+    int dist_average = true,
+    int threads = 8
 );
 
 #endif // MVE_H
